#lang racket

;;; adt-polynome.rkt -- version etudiant a completer !
;;; Un module contenant le type abstrait "polynome"
;;; PF2 - Nice/Valrose, Racket 6.2.1

(require "string2poly.rkt")       ; pour string->poly et poly->string
(require lang/posn)
(provide string->poly poly->string poly0 poly0? monome polynome degre coeff tete reste poly*ext
         poly+ poly- poly* poly-val poly-const poly-rac interpol)

;;; Un polynome est une suite ordonnee de monomes ((c e) ...), suivant les puissances decroissantes. 

(define p (string->poly "2x^5+x^4+2x^3-2x+3"))
(define q (string->poly "x^4-2x^3+7x^2-x"))
(printf "p --> ~a autrement dit ~a\n" p (poly->string p))
(printf "q --> ~a autrement dit ~a\n" q (poly->string q))

(define poly0 '())

(define (poly0? p)
  (null? p))

(define (monome c e)           ; (c e)
  (list c e))                  ; un monome N'EST PAS un polynome !

(define (polynome c e)         ; ((c e))
  (list (list c e)))

(define (degre mp)             ; p = ((cN eN) ...), retourne N
  (cond ((poly0? mp) -inf.0)            ; bof
        ((pair? (car mp)) (cadar mp))   ; polynom
        (else (cadr mp))))              ; monome

(printf "p est de degre ~a " (degre p))

(define (coeff mp)             ; le coefficient dominant cN
  (cond ((poly0? mp) 0)
        ((pair? (car mp)) (caar mp))   ; polynome
        (else (car mp))))              ; monome

(printf "et son coefficient dominant est ~a\n" (coeff p))

(define (tete p)               ; le monome de tete ((cN eN))
  (if (poly0? p) 
      (error "Pas de monome de tete pour le polynome nul !")
      (car p)))

(define (reste p)              ; le polynome prive de (cN eN)
  (if (poly0? p) 
      (error "Pas de reste pour le polynome nul !")
      (cdr p)))

(define (poly*ext k p)         ; loi externe
  (if (= k 0)
      poly0
      (map (lambda (m) (monome (* k (coeff m)) (degre m))) p)))

(printf "2p --> ~a\n" (poly->string (poly*ext 2 p)))

(define (poly+ p1 p2)
  (error 'poly+ "Not yet implemented !"))

(printf "p+q --> ~a\n" (poly->string (poly+ p q)))

(define (poly- p1 p2)
  (poly+ p1 (poly*ext -1 p2)))

(printf "p-q --> ~a\n" (poly->string (poly- p q)))
(printf "p-p --> ~a\n" (poly->string (poly- p p)))

(define (poly* p1 p2)
  (error 'poly* "Not yet implemented !"))

(printf "(2x-1)^2 --> ~a\n" (let ((p (string->poly "2x-1"))) 
                              (poly->string (poly* p p))))

(printf "(2x^100-1)(2x^100+1) --> ~a\n" 
        (poly->string (poly* (string->poly "2x^100-1") (string->poly "2x^100+1"))))

(define (poly-val p x)
  (error 'poly-val "Not yet implemented !"))

(printf "Valeur en x=2 de 3x^2-x+1 --> ~a\n" 
        (poly-val (string->poly "3x^2-x+1") 2))

;;; ********* Lagrange stuff *********

(define (poly-const a)     ; le polynome constant a
  (if (zero? a) poly0 `((,a 0))))    ; `((,a 0))  <==>  (list (list a 0))

(printf "Le polynome constant 3 --> ~a\n" (poly->string (poly-const 3)))

(define (poly-rac a)       ; le polynome elementaire X-a
  (if (zero? a) '((1 1)) `((1 1) (,(- a) 0))))      ; <------ la quasiquote !!

(printf "Le polynome elementaire de racine 3 --> ~a\n" (poly->string (poly-rac 3)))

(define (poly-lagrange xi Lx)           ; Li(x)=1 sur xi et 0 sur les points de Lx
  (define (iter Lx acc)
    (if (null? Lx)
        acc
        (let ((x1 (car Lx)))
          (iter (cdr Lx) (poly* (poly*ext (/ (- xi x1)) (poly-rac x1)) acc)))))
  (iter Lx (poly-const 1)))

(printf "(poly-lagrange 2 '(3 5)) --> ~a\n" (poly->string (poly-lagrange 2 '(3 5))))

(define (interpol Lpoints)           ; Lpoints = une liste de posn
  (let ((Lx (map posn-x Lpoints)))   ; la liste de toutes les abscisses
    (define (supprimer x L)          ; on sait que x est dans L
      (error 'supprimer "Not yet implemented !"))
    (define (iter Lpts acc)
      (error 'iter "Not yet implemented !"))
    (error 'interpol "Not yet implemented !")))

(define P (interpol (list (make-posn 3 -2) (make-posn 5 0) (make-posn 0 4))))
(printf "Polynome passant par les points (3,-2), (5,0), (0,4) --> ~a\n" (poly->string P))
(printf "Verification : valeur de ce polynome pour x=3,5,0 --> ~a\n" (map (lambda (x) (poly-val P x)) '(3 5 0)))

