#lang racket/gui

;;; lagrange-gui.rkt
;;; PF2, Nice/Valrose, Racket 6.2.1

(require "adt-polynome.rkt")
(require lang/posn)

(define N 5)                             ; nombre de points
(define SIZE 500)                        ; taille du canvas
(define STEP (quotient SIZE (+ N 1)))    ; espacement en x des points
(define LPTS '?)
(define POLY '?)

(define (random-points!)                 ; une liste de n points aleatoires espaces de dx=STEP
  (set! LPTS (build-list N (lambda (i) (make-posn (* (+ i 1.0) STEP) (random SIZE)))))
  (set! POLY (interpol LPTS)))

(random-points!)

(define PEN-LINE (make-object pen% "blue" 2 'solid))
(define PEN-POINT (make-object pen% "red" 8 'solid))

(define FRAME 
  (new frame% 
       (label "Lagrange")
       (x 10) 
       (y 10)
       (stretchable-width #f) 
       (stretchable-height #f)))

(define CANVAS 
  (new canvas% 
       (parent FRAME) 
       (min-width SIZE) 
       (min-height SIZE)
       (paint-callback (lambda (c dc)
                         (draw-lagrange dc)))))

(define BUTTON-NEW 
  (new button% 
       (parent FRAME) 
       (label "New") 
       (style '(border))
       (callback (lambda (b evt)
                   (random-points!)
                   (printf "\n------\nPolynome de Lagrange : ~a\n" (poly->string POLY))
                   (printf "Points : ~a\n" LPTS)
                   (send CANVAS refresh-now)))))

(define (draw-lagrange dc)
  (define (draw-points)
    (let ((L LPTS))        ; juste un pointeur sur LPTS, pour ne pas modifier LPTS !
      (for ([i (in-range 1 (+ N 1))])   ; LPTS est de longueur N
        (send dc draw-point (posn-x (car L)) (posn-y (car L)))
        (set! L (cdr L)))))
  (define (draw-poly)
    (for ([x (in-range STEP (+ 1 (* N STEP)))])   ; by 1
      (send dc draw-line x (poly-val POLY x) (+ x 1) (poly-val POLY (+ x 1)))))
  (send dc clear)
  (send dc set-pen PEN-POINT)
  (draw-points)
  (send dc set-pen PEN-LINE)
  (draw-poly))

(send FRAME show #t)
